@@ -19,10 +19,10 @@
 import dataclasses
 import logging
 import os
-from os.path import basename, dirname
+from os.path import basename, dirname, join, exists
 import sys
 from dataclasses import dataclass, field
-from typing import Callable, Dict, Optional, List, Union
+from typing import Callable, Dict, Optional, List, Union, Any
 from filelock import FileLock
 import time
 import tempfile
@@ -44,7 +44,8 @@ from transformers.data.processors.utils import DataProcessor, InputExample, Inpu
 from transformers import ALL_PRETRAINED_CONFIG_ARCHIVE_MAP
 from transformers.optimization import AdamW, get_scheduler
 from transformers.trainer_pt_utils import get_parameter_names
-from transformers.file_utils import hf_bucket_url, CONFIG_NAME
+from transformers.file_utils import CONFIG_NAME
+from huggingface_hub import hf_hub_url
 
 from .cnlp_processors import cnlp_processors, cnlp_output_modes, cnlp_compute_metrics, tagging, relex, classification
 from .cnlp_data import ClinicalNlpDataset, DataTrainingArguments
@@ -69,7 +70,9 @@ logger = logging.getLogger(__name__)
 @dataclass
 class CnlpTrainingArguments(TrainingArguments):
     """
-    Additional arguments specific to this class
+    Additional arguments specific to this class.
+    See all possible arguments in :class:`transformers.TrainingArguments`
+    or by passing the ``--help`` flag to this script.
     """
     evals_per_epoch: Optional[int] = field(
         default = -1, metadata={"help": "Number of times to evaluate and possibly save model per training epoch (allows for a lazy kind of early stopping)"}
@@ -91,6 +94,7 @@ class CnlpTrainingArguments(TrainingArguments):
 class ModelArguments:
     """
     Arguments pertaining to which model/config/tokenizer we are going to fine-tune from.
+    See all possible arguments by passing the ``--help`` flag to this script.
     """
     model: Optional[str] = field( default='cnlpt', 
         metadata={'help': "Model type", 'choices':cnlpt_models}
@@ -113,15 +117,62 @@ class ModelArguments:
     token: bool = field(
         default=False, metadata={"help": "Classify over an actual token rather than the [CLS] ('<s>') token -- requires that the tokens to be classified are surrounded by <e>/</e> tokens"}
     )
+
+    # NxN relation classifier-specific arguments
     num_rel_feats: Optional[int] = field(
         default=12, metadata={"help": "Number of features/attention heads to use in the NxN relation classifier"}
     )
     head_features: Optional[int] = field(
         default=64, metadata={"help": "Number of parameters in each attention head in the NxN relation classifier"}
     )
+
+    # CNN-specific arguments
+    cnn_embed_dim: Optional[int] = field(
+        default=100,
+        metadata={
+            'help': "For the CNN baseline model, the size of the word embedding space."
+        }
+    )
+    cnn_num_filters: Optional[int] = field(
+        default=25,
+        metadata={
+            'help': (
+                'For the CNN baseline model, the number of '
+                'convolution filters to use for each filter size.'
+            )
+        }
+    )
+
+    cnn_filter_sizes: Optional[List[int]] = field(
+        default_factory=lambda: [1, 2, 3],
+        metadata={
+            "help": (
+                "For the CNN baseline model, a space-separated list "
+                "of size(s) of the filters (kernels)"
+            )
+        }
+    )
+
+    # LSTM-specific arguments
+    lstm_embed_dim: Optional[int] = field(
+        default=100,
+        metadata={
+            'help': "For the LSTM baseline model, the size of the word embedding space."
+        }
+    )
+    lstm_hidden_size: Optional[int] = field(
+        default=100,
+        metadata={
+            "help": "For the LSTM baseline model, the hidden size of the LSTM layer"
+        }
+    )
+
+    # Multi-task classifier-specific arguments
     use_prior_tasks: bool = field(
         default=False, metadata={"help": "In the multi-task setting, incorporate the logits from the previous tasks into subsequent representation layers. This will be done in the task order specified in the command line."}
     )
+
+    # Hierarchical Transformer-specific arguments
     hier_num_layers: Optional[int] = field(
         default=2,
         metadata={
@@ -170,25 +221,40 @@ class ModelArguments:
 
 
 def is_pretrained_model(model_name):
-    # check if it's a built-in pre-trained config:
-    if model_name in ALL_PRETRAINED_CONFIG_ARCHIVE_MAP:
-        return True
-    
     # check if it's a model on the huggingface model hub:
-    url = hf_bucket_url(model_name, CONFIG_NAME)
+    url = hf_hub_url(model_name, CONFIG_NAME)
     r = requests.head(url)
     if r.status_code == 200:
         return True
 
     return False
 
-def main():
-    # See all possible arguments in src/transformers/training_args.py
-    # or by passing the --help flag to this script.
-    # We now keep distinct sets of args, for a cleaner separation of concerns.
+def main(json_file=None, json_obj=None):
+    """
+    See all possible arguments in :class:`transformers.TrainingArguments`
+    or by passing the --help flag to this script.
+
+    We now keep distinct sets of args, for a cleaner separation of concerns.
+
+    :param typing.Optional[str] json_file: if passed, a path to a JSON file
+        to use as the model, data, and training arguments instead of
+        retrieving them from the CLI (mutually exclusive with ``json_obj``)
+    :param typing.Optional[dict] json_obj: if passed, a JSON dictionary
+        to use as the model, data, and training arguments instead of
+        retrieving them from the CLI (mutually exclusive with ``json_file``)
+    :rtype: typing.Dict[str, typing.Dict[str, typing.Any]]
+    :return: the evaluation results (will be empty if ``--do_eval`` not passed)
+    """
     parser = HfArgumentParser((ModelArguments, DataTrainingArguments, CnlpTrainingArguments))
 
-    if len(sys.argv) == 2 and sys.argv[1].endswith(".json"):
+    if json_file is not None and json_obj is not None:
+        raise ValueError('cannot specify json_file and json_obj')
+
+    if json_file is not None:
+        model_args, data_args, training_args = parser.parse_json_file(json_file=json_file)
+    elif json_obj is not None:
+        model_args, data_args, training_args = parser.parse_dict(json_obj)
+    elif len(sys.argv) == 2 and sys.argv[1].endswith(".json"):
         # If we pass only one argument to the script and it's the path to a json file,
         # let's parse it to get our arguments.
         model_args, data_args, training_args = parser.parse_json_file(json_file=os.path.abspath(sys.argv[1]))
@@ -289,9 +355,26 @@ def main():
     pretrained = False
 
     if model_name == 'cnn':
-        model = CnnSentenceClassifier(len(tokenizer), num_labels_list=num_labels)
+        model = CnnSentenceClassifier(len(tokenizer), 
+                                      num_labels_list=num_labels,
+                                      embed_dims=model_args.cnn_embed_dim,
+                                      num_filters=model_args.cnn_num_filters,
+                                      filters=model_args.cnn_filter_sizes,
+                                      )
+        # Check if the caller specified a saved model to load (e.g., for an inference-only run)
+        model_path = join(model_args.encoder_name, 'pytorch_model.bin')
+        if exists(model_path):
+            model.load_state_dict(torch.load(model_path))
     elif model_name == 'lstm':
-        model = LstmSentenceClassifier(len(tokenizer), num_labels_list=num_labels)
+        model = LstmSentenceClassifier(len(tokenizer),
+                                       num_labels_list=num_labels,
+                                       embed_dims=model_args.lstm_embed_dim,
+                                       hidden_size=model_args.lstm_hidden_size,
+                                       )
+        # Check if the caller specified a saved model to load (e.g., for an inference-only run)
+        model_path = join(model_args.encoder_name, 'pytorch_model.bin')
+        if exists(model_path):
+            model.load_state_dict(torch.load(model_path))
     elif model_name == 'hier':
         # encoder_config = AutoConfig.from_pretrained(
         #     model_args.config_name if model_args.config_name else model_args.encoder_name,
@@ -489,12 +572,12 @@ def main():
 
             if not model is None:
                 if not hasattr(model, 'best_score') or one_score > model.best_score:
-                    if pretrained:
-                        trainer.save_model()
                     # For convenience, we also re-save the tokenizer to the same directory,
                     # so that you can share your model easily on huggingface.co/models =)
                     if trainer.is_world_process_zero():
-                        tokenizer.save_pretrained(training_args.output_dir)
+                        if training_args.do_train:
+                            trainer.save_model()
+                            tokenizer.save_pretrained(training_args.output_dir)
                         for task_ind,task_name in enumerate(metrics):
                             with open(output_eval_file, "w") as writer:
                                 logger.info("***** Eval results for task %s *****" % (task_name))
@@ -524,12 +607,11 @@ def main():
         )
 
         if not hasattr(model, 'best_score'):
-            if pretrained:
+            # For convenience, we also re-save the tokenizer to the same directory,
+            # so that you can share your model easily on huggingface.co/models =)
+            if trainer.is_world_process_zero():
                 trainer.save_model()
-                # For convenience, we also re-save the tokenizer to the same directory,
-                # so that you can share your model easily on huggingface.co/models =)
-                if trainer.is_world_process_zero():
-                    tokenizer.save_pretrained(training_args.output_dir)
+                tokenizer.save_pretrained(training_args.output_dir)
 
     # Evaluation
     eval_results = {}
